/*
 *  TOPPERS/JSP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Just Standard Profile Kernel
 * 
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 * 
 *  上記著作権者は，以下の (1)〜(4) の条件か，Free Software Foundation 
 *  によって公表されている GNU General Public License の Version 2 に記
 *  述されている条件を満たす場合に限り，本ソフトウェア（本ソフトウェア
 *  を改変したものを含む．以下同じ）を使用・複製・改変・再配布（以下，
 *  利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，その適用可能性も
 *  含めて，いかなる保証も行わない．また，本ソフトウェアの利用により直
 *  接的または間接的に生じたいかなる損害に関しても，その責任を負わない．
 * 
 *  @(#) $Id: cpu_config.h,v 1.1 2009/01/31 05:27:37 suikan Exp $
 */

/*
 *  プロセッサ依存モジュール（M68040用）
 *
 *  このインクルードファイルは，t_config.h のみからインクルードされる．
 *  他のファイルから直接インクルードしてはならない．
 */

#ifndef _CPU_CONFIG_H_
#define _CPU_CONFIG_H_

/*
 *  カーネルの内部識別名のリネーム
 */
#include <cpu_rename.h>

/*
 *  プロセッサの特殊命令のインライン関数定義
 */
#ifndef _MACRO_ONLY
#include <cpu_insn.h>
#endif /* _MACRO_ONLY */

/*
 *  chg_ipm/get_ipm をサポートするかどうかの定義
 */
#define SUPPORT_CHG_IPM

/*
 *  TCB 中のフィールドのビット幅の定義
 *
 *  cpu_context.h に入れる方がエレガントだが，参照の依存性の関係で，
 *  cpu_context.h には入れられない．
 */
#define TBIT_TCB_TSTAT      8   /* tstat フィールドのビット幅 */
#define TBIT_TCB_PRIORITY   8   /* priority フィールドのビット幅 */

#ifndef _MACRO_ONLY
/*
 *  タスクコンテキストブロックの定義
 */
typedef struct task_context_block {
    VP  msp;        /* スタックポインタ */
    FP  pc;     /* プログラムカウンタ */
} CTXB;

/*
 *  割込みマスク操作ライブラリ
 *
 *  割込みマスク（intmask）は，IPM（Interrupt Priority Mask）を8ビット
 *  左にシフトしたものである．
 */

/*
 *  現在の割込みマスクの読出し
 */
Inline UH
current_intmask()
{
    return(current_sr() & 0x0700);
}

/*
 *  割込みマスクの設定
 */
Inline void
set_intmask(UH intmask)
{
    set_sr((current_sr() & ~0x0700) | intmask);
}

/*
 *  システム状態参照
 */

Inline BOOL
sense_context()
{
    return((current_sr() & 0x1000) == 0);
}

Inline BOOL
sense_lock()
{
    return(current_intmask() == 0x0700);
}

#define t_sense_lock    sense_lock
#define i_sense_lock    sense_lock

/*
 *  CPUロックとその解除（タスクコンテキスト用）
 *
 *  task_intmask は，chg_ipm をサポートするための変数．chg_ipm をサポー
 *  トしない場合には，task_intmask が常に 0 になっていると考えればよい．
 */

#ifdef SUPPORT_CHG_IPM
extern UH   task_intmask;   /* タスクコンテキストでの割込みマスク */
#endif /* SUPPORT_CHG_IPM */

Inline void
t_lock_cpu()
{
    disint();
}

Inline void
t_unlock_cpu()
{
#ifdef SUPPORT_CHG_IPM
    /*
     *  t_unlock_cpu が呼び出されるのは CPUロック状態のみであるた
     *  め，処理の途中で task_intmask が書き換わることはない．
     */
    set_intmask(task_intmask);
#else /* SUPPORT_CHG_IPM */
    enaint();
#endif /* SUPPORT_CHG_IPM */
}

/*
 *  CPUロックとその解除（非タスクコンテキスト用）
 */

extern UH   int_intmask;    /* 非タスクコンテキストでの割込みマスク */

Inline void
i_lock_cpu()
{
    UH  intmask;

    /*
     *  一時変数 intmask を使っているのは，current_intmask() を呼
     *  んだ直後に割込みが発生し，起動された割込みハンドラ内で
     *  int_intmask が変更される可能性があるためである．
     */
    intmask = current_intmask();
    disint();
    int_intmask = intmask;
}

Inline void
i_unlock_cpu()
{
    set_intmask(int_intmask);
}

/*
 *  タスクディスパッチャ
 */

/*
 *  最高優先順位タスクへのディスパッチ（cpu_support.S）
 *
 *  dispatch は，タスクコンテキストから呼び出されたサービスコール処理
 *  内で，CPUロック状態で呼び出さなければならない．
 */
extern void dispatch(void);

/*
 *  現在のコンテキストを捨ててディスパッチ（cpu_support.S）
 *
 *  exit_and_dispatch は，CPUロック状態で呼び出さなければならない．
 */
extern void exit_and_dispatch(void);

/*
 *  割込みハンドラ／CPU例外ハンドラの設定
 */

/*
 *  例外ベクタテーブルの構造の定義
 */
typedef struct exc_vector_entry {
    FP  exchdr;         /* 例外ハンドラの起動番地 */
} EXCVE;

/*
 *  割込みハンドラの設定
 *
 *  ベクトル番号 inhno の割込みハンドラの起動番地を inthdr に設定する．
 */
Inline void
define_inh(INHNO inhno, FP inthdr)
{
    EXCVE   *excvt;

#ifdef EXCVT_KERNEL
    /*
     *  EXCVT_KERNEL が定義されている時は，初期化処理の中で VBR を 
     *  EXCVT_KERNEL に設定するので，EXCVT_KERNEL を使う．
     */
    excvt = (EXCVE *) EXCVT_KERNEL;
#else /* EXCVT_KERNEL */
    excvt = (EXCVE *) current_vbr();
#endif /* EXCVT_KERNEL */
    excvt[inhno].exchdr = inthdr;
}

/*
 *  CPU例外ハンドラの設定
 *
 *  ベクトル番号 excno のCPU例外ハンドラの起動番地を exchdr に設定する．
 */
Inline void
define_exc(EXCNO excno, FP exchdr)
{
    define_inh(excno, exchdr);
}

/*
 *  割込みハンドラ／CPU例外ハンドラの出入口処理
 */

/*
 *  割込みハンドラの出入口処理の生成マクロ
 *
 *  reqflg をチェックする前に割込みを禁止しないと，reqflg をチェック後
 *  に起動された割込みハンドラ内でディスパッチが要求された場合に，ディ
 *  スパッチされない．
 */

#define INTHDR_ENTRY(inthdr)        \
extern void inthdr##_entry(void);   \
asm(".text              \n" \
#inthdr "_entry:            \n" \
"   movem.l %d0-%d1/%a0-%a1, -(%sp) \n" /* スクラッチレジスタを保存 */ \
"   jsr " #inthdr "         \n" /* 割込みハンドラを呼び出す */ \
"   movem.l (%sp)+, %d0-%d1/%a0-%a1 \n" /* スクラッチレジスタを復帰 */ \
"   btst.b #4, (%sp)        \n" /* 戻り先が割込みモードなら */ \
"   jbeq 1f             \n" /*           すぐにリターン */ \
"   or.w #0x0700, %sr       \n" /* 割込み禁止 */ \
"   tst.l _kernel_reqflg        \n" /* reqflg が TRUE であれば */ \
"   jbne _kernel_ret_int        \n" /*              ret_int へ */ \
"1: rte             \n")

#define INT_ENTRY(inthdr)   inthdr##_entry

/*
 *  CPU例外ハンドラの出入口処理の生成マクロ
 *
 *  CPU例外ハンドラは，非タスクコンテキストで実行する．そのため，CPU例
 *  外ハンドラを呼び出す前に割込みモードに移行し，リターンしてきた後に
 *  元のモードに戻す．元のモードに戻すために，割込みモードに移行する前
 *  の SR を割込みスタック上に保存する．CPU例外がタスクコンテキストで
 *  発生し，reqflg が TRUE になった時に，ret_exc へ分岐する．
 *  reqflg をチェックする前に割込みを禁止しないと，reqflg をチェック後
 *  に起動された割込みハンドラ内でディスパッチが要求された場合に，ディ
 *  スパッチされない．
 */

#define EXCHDR_ENTRY(exchdr)        \
extern void exchdr##_entry(VP sp);  \
asm(".text              \n" \
#exchdr "_entry:            \n" \
"   movem.l %d0-%d1/%a0-%a1, -(%sp) \n" /* スクラッチレジスタを保存 */ \
"   lea.l 16(%sp), %a0      \n" /* 例外フレームの先頭を A0 に */ \
"   move.w %sr, %d0         \n" /* SR を D0 に */ \
"   and.w #~0x1000, %sr     \n" /* 割込みモード */ \
"   move.l %d0, -(%sp)      \n" /* 元の SR をスタックに保存 */ \
"   move.l %a0, -(%sp)      \n" /* A0 を引数として渡す */ \
"   jsr " #exchdr "         \n" /* CPU例外ハンドラを呼び出す */ \
"   addq.l #4, %sp          \n" \
"   move.l (%sp)+, %d0      \n" \
"   and.w #0x1000, %d0      \n" /* 元が割込みモードなら */ \
"   jbeq 1f             \n" /*       すぐにリターン */ \
"   or.w #0x1700, %sr       \n" /* マスタモード・割込み禁止 */ \
"   tst.l _kernel_reqflg        \n" /* reqflg が TRUE であれば */ \
"   jbne _kernel_ret_exc        \n" /*              ret_exc へ */ \
"1: movem.l (%sp)+, %d0-%d1/%a0-%a1 \n" /* スクラッチレジスタを復帰 */ \
"   rte             \n")

#define EXC_ENTRY(exchdr)   exchdr##_entry

/*
 *  CPU例外の発生した時のシステム状態の参照
 */

/*
 *  CPU例外の発生した時のコンテキストの参照
 */
Inline BOOL
exc_sense_context(VP p_excinf)
{
    return((*((UH *) p_excinf) & 0x1000) == 0);
}

/*
 *  CPU例外の発生した時のCPUロック状態の参照
 */
Inline BOOL
exc_sense_lock(VP p_excinf)
{
    return((*((UH *) p_excinf) & 0x0700) == 0x0700);
}

/*
 *  プロセッサ依存の初期化
 */
extern void cpu_initialize(void);

/*
 *  プロセッサ依存の終了時処理
 */
extern void cpu_terminate(void);

#endif /* _MACRO_ONLY */
#endif /* _CPU_CONFIG_H_ */
